<?php

namespace Litecms\Addons\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Addons\Http\Requests\AddonsRequest;
use Litecms\Addons\Interfaces\AddonsRepositoryInterface;
use Litecms\Addons\Models\Addons;

/**
 * Resource controller class for addons.
 */
class AddonsResourceController extends BaseController
{

    /**
     * Initialize addons resource controller.
     *
     * @param type AddonsRepositoryInterface $addons
     *
     * @return null
     */
    public function __construct(AddonsRepositoryInterface $addons)
    {
        parent::__construct();
        $this->repository = $addons;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Addons\Repositories\Criteria\AddonsResourceCriteria::class);
    }

    /**
     * Display a list of addons.
     *
     * @return Response
     */
    public function index(AddonsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Addons\Repositories\Presenter\AddonsPresenter::class)
                ->$function();
        }

        $addons = $this->repository->paginate();

        return $this->response->title(trans('addons::addons.names'))
            ->view('addons::addons.index', true)
            ->data(compact('addons'))
            ->output();
    }

    /**
     * Display addons.
     *
     * @param Request $request
     * @param Model   $addons
     *
     * @return Response
     */
    public function show(AddonsRequest $request, Addons $addons)
    {

        if ($addons->exists) {
            $view = 'addons::addons.show';
        } else {
            $view = 'addons::addons.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('addons::addons.name'))
            ->data(compact('addons'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new addons.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AddonsRequest $request)
    {

        $addons = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('addons::addons.name')) 
            ->view('addons::addons.create', true) 
            ->data(compact('addons'))
            ->output();
    }

    /**
     * Create new addons.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AddonsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $addons                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('addons::addons.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('addons/addons/' . $addons->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/addons/addons'))
                ->redirect();
        }

    }

    /**
     * Show addons for editing.
     *
     * @param Request $request
     * @param Model   $addons
     *
     * @return Response
     */
    public function edit(AddonsRequest $request, Addons $addons)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('addons::addons.name'))
            ->view('addons::addons.edit', true)
            ->data(compact('addons'))
            ->output();
    }

    /**
     * Update the addons.
     *
     * @param Request $request
     * @param Model   $addons
     *
     * @return Response
     */
    public function update(AddonsRequest $request, Addons $addons)
    {
        try {
            $attributes = $request->all();

            $addons->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('addons::addons.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('addons/addons/' . $addons->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('addons/addons/' . $addons->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the addons.
     *
     * @param Model   $addons
     *
     * @return Response
     */
    public function destroy(AddonsRequest $request, Addons $addons)
    {
        try {

            $addons->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('addons::addons.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('addons/addons/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('addons/addons/' . $addons->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple addons.
     *
     * @param Model   $addons
     *
     * @return Response
     */
    public function delete(AddonsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('addons::addons.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('addons/addons'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/addons/addons'))
                ->redirect();
        }

    }

    /**
     * Restore deleted addons.
     *
     * @param Model   $addons
     *
     * @return Response
     */
    public function restore(AddonsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('addons::addons.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/addons/addons'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/addons/addons/'))
                ->redirect();
        }

    }

}
